@@ -1,3 +1,4 @@
+from __future__ import annotations
 # built-in
 from collections import defaultdict
 from itertools import groupby
@@ -6,20 +7,19 @@ from itertools import groupby
 from .base import Base as _Base, BaseSimilarity as _BaseSimilarity
 
 
-try:
-    from itertools import zip_longest
-except ImportError:
-    from itertools import izip_longest as zip_longest
+from itertools import zip_longest
+from typing import Any, Iterator, Sequence, TypeVar
 try:
     import numpy
 except ImportError:
-    numpy = None
+    numpy = None  # type: ignore[assignment]
 
 
 __all__ = [
     'MRA', 'Editex',
     'mra', 'editex',
 ]
+T = TypeVar('T')
 
 
 class MRA(_BaseSimilarity):
@@ -28,11 +28,11 @@ class MRA(_BaseSimilarity):
     https://github.com/Yomguithereal/talisman/blob/master/src/metrics/mra.js
     """
 
-    def maximum(self, *sequences):
+    def maximum(self, *sequences: str) -> int:
         sequences = [list(self._calc_mra(s)) for s in sequences]
         return max(map(len, sequences))
 
-    def _calc_mra(self, word):
+    def _calc_mra(self, word: str) -> str:
         if not word:
             return word
         word = word.upper()
@@ -43,7 +43,7 @@ class MRA(_BaseSimilarity):
             return word[:3] + word[-3:]
         return word
 
-    def __call__(self, *sequences):
+    def __call__(self, *sequences: str) -> int:
         if not all(sequences):
             return 0
         sequences = [list(self._calc_mra(s)) for s in sequences]
@@ -61,6 +61,7 @@ class MRA(_BaseSimilarity):
                     new_sequences.append(chars)
             new_sequences = map(list, zip(*new_sequences))
             # update sequences
+            ss: Iterator[tuple[Any, Any]]
             ss = zip_longest(new_sequences, sequences, fillvalue=list())
             sequences = [s1 + s2[minlen:] for s1, s2 in ss]
             # update lengths
@@ -79,7 +80,7 @@ class Editex(_Base):
     https://github.com/chrislit/blob/master/abydos/distance/_editex.py
     https://habr.com/ru/post/331174/ (RUS)
     """
-    groups = (
+    groups: tuple[frozenset[str], ...] = (
         frozenset('AEIOUY'),
         frozenset('BP'),
         frozenset('CKQ'),
@@ -93,8 +94,16 @@ class Editex(_Base):
     )
     ungrouped = frozenset('HW')  # all letters in alphabet that not presented in `grouped`
 
-    def __init__(self, local=False, match_cost=0, group_cost=1, mismatch_cost=2,
-                 groups=None, ungrouped=None, external=True):
+    def __init__(
+        self,
+        local: bool = False,
+        match_cost: int = 0,
+        group_cost: int = 1,
+        mismatch_cost: int = 2,
+        groups: tuple[frozenset[str], ...] = None,
+        ungrouped: frozenset[str] = None,
+        external: bool = True,
+    ) -> None:
         # Ensure that match_cost <= group_cost <= mismatch_cost
         self.match_cost = match_cost
         self.group_cost = max(group_cost, self.match_cost)
@@ -109,14 +118,10 @@ class Editex(_Base):
             self.ungrouped = ungrouped
         self.grouped = frozenset.union(*self.groups)
 
-        # backward compat
-        if hasattr(self, 'letter_groups'):
-            self.groups = self.letter_groups
-
-    def maximum(self, *sequences):
+    def maximum(self, *sequences: Sequence) -> int:
         return max(map(len, sequences)) * self.mismatch_cost
 
-    def r_cost(self, *elements):
+    def r_cost(self, *elements: str) -> int:
         if self._ident(*elements):
             return self.match_cost
         if any(map(lambda x: x not in self.grouped, elements)):
@@ -126,12 +131,12 @@ class Editex(_Base):
                 return self.group_cost
         return self.mismatch_cost
 
-    def d_cost(self, *elements):
+    def d_cost(self, *elements: str) -> int:
         if not self._ident(*elements) and elements[0] in self.ungrouped:
             return self.group_cost
         return self.r_cost(*elements)
 
-    def __call__(self, s1, s2):
+    def __call__(self, s1: str, s2: str) -> float:
         result = self.quick_answer(s1, s2)
         if result is not None:
             return result
@@ -145,6 +150,7 @@ class Editex(_Base):
         s2 = ' ' + s2.upper()
         len_s1 = len(s1) - 1
         len_s2 = len(s2) - 1
+        d_mat: Any
         if numpy:
             d_mat = numpy.zeros((len_s1 + 1, len_s2 + 1), dtype=int)
         else:
